#ifndef _LINUX_V4L2_CONTROLS_H
#define _LINUX_V4L2_CONTROLS_H

#ifdef __cplusplus
extern "C" {
#endif

// Control classes
#define V4L2_CTRL_CLASS_USER		0x00980000	/* Old-style 'user' controls */
#define V4L2_CTRL_CLASS_CAMERA		0x009a0000	/* Camera class controls */

// User-class control IDs
#define V4L2_CID_BASE			(V4L2_CTRL_CLASS_USER | 0x900)
#define V4L2_CID_BRIGHTNESS		(V4L2_CID_BASE + 0)
#define V4L2_CID_CONTRAST		(V4L2_CID_BASE + 1)
#define V4L2_CID_SATURATION		(V4L2_CID_BASE + 2)
#define V4L2_CID_HUE			(V4L2_CID_BASE + 3)

/*  Camera class control IDs */
#define V4L2_CID_CAMERA_CLASS_BASE	(V4L2_CTRL_CLASS_CAMERA | 0x900)
#define V4L2_CID_CAMERA_CLASS		(V4L2_CTRL_CLASS_CAMERA | 1)

#define V4L2_CID_EXPOSURE_AUTO			(V4L2_CID_CAMERA_CLASS_BASE + 1)
enum  v4l2_exposure_auto_type {
	V4L2_EXPOSURE_AUTO = 0,
	V4L2_EXPOSURE_MANUAL = 1,
	V4L2_EXPOSURE_SHUTTER_PRIORITY = 2,
	V4L2_EXPOSURE_APERTURE_PRIORITY = 3
};
#define V4L2_CID_EXPOSURE_ABSOLUTE		(V4L2_CID_CAMERA_CLASS_BASE + 2)
#define V4L2_CID_EXPOSURE_AUTO_PRIORITY		(V4L2_CID_CAMERA_CLASS_BASE + 3)
#define V4L2_CID_AUTO_EXPOSURE_BIAS		(V4L2_CID_CAMERA_CLASS_BASE + 19)
#define V4L2_CID_ISO_SENSITIVITY		(V4L2_CID_CAMERA_CLASS_BASE + 23)
#define V4L2_CID_ISO_SENSITIVITY_AUTO		(V4L2_CID_CAMERA_CLASS_BASE + 24)
enum v4l2_iso_sensitivity_auto_type {
	V4L2_ISO_SENSITIVITY_MANUAL		= 0,
	V4L2_ISO_SENSITIVITY_AUTO		= 1,
};

/* User-class control IDs */

#define V4L2_CID_BASE			(V4L2_CTRL_CLASS_USER | 0x900)
#define V4L2_CID_USER_BASE		V4L2_CID_BASE
#define V4L2_CID_USER_CLASS		(V4L2_CTRL_CLASS_USER | 1)
#define V4L2_CID_BRIGHTNESS		(V4L2_CID_BASE + 0)
#define V4L2_CID_CONTRAST		(V4L2_CID_BASE + 1)
#define V4L2_CID_SATURATION		(V4L2_CID_BASE + 2)
#define V4L2_CID_HUE			(V4L2_CID_BASE + 3)
#define V4L2_CID_AUDIO_VOLUME		(V4L2_CID_BASE + 5)
#define V4L2_CID_AUDIO_BALANCE		(V4L2_CID_BASE + 6)
#define V4L2_CID_AUDIO_BASS		(V4L2_CID_BASE + 7)
#define V4L2_CID_AUDIO_TREBLE		(V4L2_CID_BASE + 8)
#define V4L2_CID_AUDIO_MUTE		(V4L2_CID_BASE + 9)
#define V4L2_CID_AUDIO_LOUDNESS		(V4L2_CID_BASE + 10)
#define V4L2_CID_BLACK_LEVEL		(V4L2_CID_BASE + 11) /* Deprecated */
#define V4L2_CID_AUTO_WHITE_BALANCE	(V4L2_CID_BASE + 12)
#define V4L2_CID_DO_WHITE_BALANCE	(V4L2_CID_BASE + 13)
#define V4L2_CID_RED_BALANCE		(V4L2_CID_BASE + 14)
#define V4L2_CID_BLUE_BALANCE		(V4L2_CID_BASE + 15)
#define V4L2_CID_GAMMA			(V4L2_CID_BASE + 16)
#define V4L2_CID_WHITENESS		(V4L2_CID_GAMMA) /* Deprecated */
#define V4L2_CID_EXPOSURE		(V4L2_CID_BASE + 17)
#define V4L2_CID_AUTOGAIN		(V4L2_CID_BASE + 18)
#define V4L2_CID_GAIN			(V4L2_CID_BASE + 19)
#define V4L2_CID_HFLIP			(V4L2_CID_BASE + 20)
#define V4L2_CID_VFLIP			(V4L2_CID_BASE + 21)

#define V4L2_CID_POWER_LINE_FREQUENCY	(V4L2_CID_BASE + 24)
enum v4l2_power_line_frequency {
	V4L2_CID_POWER_LINE_FREQUENCY_DISABLED	= 0,
	V4L2_CID_POWER_LINE_FREQUENCY_50HZ	= 1,
	V4L2_CID_POWER_LINE_FREQUENCY_60HZ	= 2,
	V4L2_CID_POWER_LINE_FREQUENCY_AUTO	= 3,
};
#define V4L2_CID_HUE_AUTO			(V4L2_CID_BASE + 25)
#define V4L2_CID_WHITE_BALANCE_TEMPERATURE	(V4L2_CID_BASE + 26)
#define V4L2_CID_SHARPNESS			(V4L2_CID_BASE + 27)
#define V4L2_CID_BACKLIGHT_COMPENSATION		(V4L2_CID_BASE + 28)
#define V4L2_CID_CHROMA_AGC                     (V4L2_CID_BASE + 29)
#define V4L2_CID_COLOR_KILLER                   (V4L2_CID_BASE + 30)
#define V4L2_CID_COLORFX			(V4L2_CID_BASE + 31)
enum v4l2_colorfx {
	V4L2_COLORFX_NONE			= 0,
	V4L2_COLORFX_BW				= 1,
	V4L2_COLORFX_SEPIA			= 2,
	V4L2_COLORFX_NEGATIVE			= 3,
	V4L2_COLORFX_EMBOSS			= 4,
	V4L2_COLORFX_SKETCH			= 5,
	V4L2_COLORFX_SKY_BLUE			= 6,
	V4L2_COLORFX_GRASS_GREEN		= 7,
	V4L2_COLORFX_SKIN_WHITEN		= 8,
	V4L2_COLORFX_VIVID			= 9,
	V4L2_COLORFX_AQUA			= 10,
	V4L2_COLORFX_ART_FREEZE			= 11,
	V4L2_COLORFX_SILHOUETTE			= 12,
	V4L2_COLORFX_SOLARIZATION		= 13,
	V4L2_COLORFX_ANTIQUE			= 14,
	V4L2_COLORFX_SET_CBCR			= 15,
};
#define V4L2_CID_AUTOBRIGHTNESS			(V4L2_CID_BASE + 32)
#define V4L2_CID_BAND_STOP_FILTER		(V4L2_CID_BASE + 33)

#define V4L2_CID_ROTATE				(V4L2_CID_BASE + 34)
#define V4L2_CID_BG_COLOR			(V4L2_CID_BASE + 35)

#ifdef __cplusplus
}
#endif

#endif // _LINUX_V4L2_CONTROLS_H
